from libc.stdlib cimport malloc, free
from libc.string cimport memset
import random

cdef extern from "math.h":
    double sqrt(double theta)

cdef _c_rdp(double *x, double *y, int n, char* mask, double epsilon, int min_points, int* order, double *distances):
    stk = [(0, n - 1)]
    cdef int st, ed, index
    cdef double dmax, p0, p1, p2,dis
    cdef int num_points = n
    cdef int j = 0
    while stk:
        st, ed = stk.pop()
        dis = sqrt((y[st]-y[ed]) * (y[st]-y[ed]) + (x[st]-x[ed]) * (x[st]-x[ed]))
        p0 = y[st] - y[ed]
        p1 = x[st] - x[ed]
        p2 = x[st] * y[ed] - y[st] * x[ed]
        dmax = 0.0
        index = st
        for i from st < i < ed:
            d = abs(p0 * x[i] - p1 * y[i] + p2) / dis
            if d > dmax:
                index = i
                dmax = d
        if dmax > epsilon:
            stk.append((index, ed))
            stk.append((st, index))
            distances[index] = dmax
            j += 1
        elif (num_points - (ed -st)) > min_points:
            for i from st < i < ed:
                if mask[i]:
                    num_points -= 1
                mask[i] = 0
               

def rdp(points, double epsilon=0, int min_points=2):
    cdef int n
    n = len(points)
    cdef double *x = <double*> malloc(n * sizeof(double))
    cdef double *y = <double*> malloc(n * sizeof(double))
    cdef double *distances = <double*> malloc(n * sizeof(double))
    cdef char *mask = <char*> malloc(n)
    cdef int *order = <int*> malloc(n * sizeof(int))
    memset(mask, 1, n)
    for i from 0<=i<n:
        p = points[i]
        x[i] = p[0]
        y[i] = p[1]
    _c_rdp(x, y, n, mask, epsilon, min_points, order, distances)
    res = []
    o = []
    d = []
    for i from 0<=i<n:
        if mask[i]:
            res.append(points[i])
        d.append(distances[i])
    free(x)
    free(y)
    return res, o, d


from Cython.Build import cythonize
from setuptools import Extension, find_packages, setup

extensions = [
    Extension("watsoncrdp", ["watson-crdp.pyx"]),
]

try:
    from Cython.Build import cythonize

    extensions = cythonize(extensions, compiler_directives={"language_level": 3})
except ImportError:
    pass


setup(
    name="watsoncrdp",
    packages=(find_packages()),
    install_requires=[],
    extras_require=dict(dev=["cython",],),
    ext_modules=extensions,
)



from watsoncrdp import rdp as wrdp

# -*- coding: utf-8 -*-
"""
Created on Sat Jan 18 08:00:00 2019
@author: omu
Correction of raw sensordata in CALYPSO
"""

import json
import os
import sys
import time

import numpy as np
import pandas as pd
import psycopg2
import psycopg2.extras

CONNECTION = None
schema = "sensor"


def get_connection():
    global CONNECTION
    if not CONNECTION:
        CONNECTION = psycopg2.connect(
            dbname="watsonc",
            user=os.environ["DB_USER"],
            password=os.environ["DB_PASSWORD"],
            host=os.environ["DB_HOST"],
            options=f"-c search_path={schema}",
        )
    return CONNECTION


def get_all_data(ts_id):
    conn = get_connection()

    statement = f"""SELECT timeofmeas, measurement FROM sensordata_corrected
        WHERE ts_id = {ts_id}"""

    data = pd.read_sql(statement, con=conn)

    statement = f"DELETE FROM sensordata_reduced where ts_id = {ts_id}"
    with conn.cursor() as curr:
        curr.execute(statement)
        conn.commit()

    data["timeofmeas"] = pd.to_datetime(data["timeofmeas"], utc=True).dt.tz_convert(
        "Europe/Copenhagen"
    )

    data = data.drop_duplicates(subset=["timeofmeas"])
    data = data.dropna()

    data = data.set_index("timeofmeas")
    data = data.sort_index(ascending=True)
    return data


data_orig = get_all_data(5)
data = data_orig.copy()
timeofmeas = data.index.astype(np.int64).tolist()
measurements = data.measurement.tolist()
points = list(zip(timeofmeas, measurements))
epsilon = 0.01
minimum = 2

start = time.time()
rd2, order, dis = wrdp(points, epsilon=epsilon, min_points=minimum)
end = time.time()
print(f"before: {len(measurements)} after: {len(rd2)} time: {end-start}")
print(order)

od = list(zip(dis, order))
od.sort(reverse=True)

start = time.time()
data_reduced = pd.DataFrame(rd2, columns=["timeofmeas", "measurement"])
data_reduced["timeofmeas"] = pd.to_datetime(
    data_reduced["timeofmeas"], utc=True
).dt.tz_convert("Europe/Copenhagen")
data_reduced = data_reduced.set_index("timeofmeas")


data_important = pd.DataFrame(
    [p for i, p in enumerate(points) if dis[i] > 0.2], columns=["timeofmeas", "measurement"]
)
data_important["timeofmeas"] = pd.to_datetime(
    data_important["timeofmeas"], utc=True
).dt.tz_convert("Europe/Copenhagen")

data_important = data_important.set_index("timeofmeas")
data_important = data_important.sort_index(ascending=True)

data_reduced.plot(backend="plotly", title="Reduced data")
data_important.plot(backend="plotly", title="Important data")

# if len(rd2) > 100000:
#     # data = data.rolling(window=5, center=True).mean().dropna()
#     data = data.resample("1H").mean()
#     timeofmeas = data.index.astype(np.int64).tolist()
#     measurements = data.measurement.tolist()
#     rd2 = wrdp(list(zip(timeofmeas, measurements)), epsilon=epsilon, min_points=minimum)

#     data_reduced = pd.DataFrame(rd2, columns=["timeofmeas", "measurement"])
#     data_reduced["timeofmeas"] = pd.to_datetime(
#         data_reduced["timeofmeas"], utc=True
#     ).dt.tz_convert("Europe/Copenhagen")
#     data_reduced = data_reduced.set_index("timeofmeas")

# print(f"before: {len(measurements)} after: {len(rd2)} time: {end-start}")

# freq = 24

# my_range = pd.date_range(
#     start=data_reduced.index.date.min(),
#     end=data_reduced.index.date.max(),
#     freq=f"{freq}H",
# ).tz_localize("Europe/Copenhagen", nonexistent="shift_forward")


# # diff = my_range.difference(data_reduced.index.date).tz_localize("Europe/Copenhagen")

# diff = pd.merge_asof(
#     pd.DataFrame(my_range + pd.Timedelta(hours=freq // 2), columns=["time"]),
#     data_reduced.reset_index(),
#     left_on="time",
#     right_on="timeofmeas",
#     direction="nearest",
#     tolerance=pd.Timedelta(hours=freq // 2),
# )

# diff = diff[diff.timeofmeas.isnull()].time

# merged = pd.merge_asof(
#     pd.DataFrame(diff, columns=["time"]),
#     data.reset_index(),
#     left_on="time",
#     right_on="timeofmeas",
#     direction="nearest",
#     tolerance=pd.Timedelta(hours=freq // 2),
# ).dropna()
# merged = merged.set_index("timeofmeas").drop(columns=["time"])

# out = pd.concat((merged, data_reduced)).sort_index()
# end = time.time()
# print(f"before: {len(rd2)} after: {len(out)} time: {end-start}")

# data.plot(x="timeofmeas", y="measurement")
# plt.show()
